# sb::App::Upgrade
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App::Upgrade;
use strict;
use sb::Driver ();
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.01';
# 0.01 [2008/11/09] $sb::DBTABLE = 2 :: inherited sb::App / added upgrade to handle callbacks
# 0.00 [2008/07/18] $sb::DBTABLE = 1 :: generated
# --------------------------------------
# // defines
sub CONTENT_TYPE   (){ 'text/plain' };
sub OUTPUT_CHARSET (){ 'utf-8' };
sub OUTPUT_CODE    (){ 'utf8' };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	$self->SUPER::initialize(
		'driver'  => undef,
		'current' => 0,
		@_
	);
	$self->generate_accessor('current','driver');
	return $self;
}
sub run
{
	my $class = shift;
	my $self = $class->new(@_);
	my $output = 'done';
	# [TODO] implement upgrade routine here
	$self->cgi->head(
		'type'    => CONTENT_TYPE,
		'charset' => OUTPUT_CHARSET,
		'length'  => length($output),
	);
	$self->output($output);
}
# --------------------------------------
# // public functions
sub upgrade
{
	my $self = shift;
	return if ( !$self->driver );
	my $err = undef;
	for (my $ver = 0; $ver < $sb::DBTABLE; $ver++ )
	{
		next if ( $ver < $self->current );
		my $change = $self->_check_list($ver);
		if ( $change->{'data'}
		 and ref($change->{'data'}) eq 'ARRAY' )
		{
			foreach ( @{$change->{'data'}} )
			{
				if ( !$self->driver->alter(@{$_}) )
				{
					warn('failed to update : ' . $_->[0]);
					$err = 1;
				}
			}
		}
		if ( $change->{'call'}
		 and ref($change->{'call'}) eq 'CODE' )
		{
			eval { &{$change->{'call'}}($self); };
			if ( $@ )
			{
				warn('failed to update : ' . $@);
				$err = 1;
			}
		}
	} # end of for ( $ver )
}
# --------------------------------------
# // private functions
sub _check_list
{
	my $self = shift;
	my $version = shift;
	my %change = (
		'data' => undef,
		'call' => undef,
	);
	if ( $version == 0 )
	{
		$change{'data'} = [
			['sb::Data::Category','disp'],
			['sb::Data::Weblog','dbset','ADD'],
		];
	}
	elsif ( $version == 1 )
	{
		$change{'data'} = [
			['sb::Data::Amazon','id'],
			['sb::Data::Category','id'],
			['sb::Data::Config','id'],
			['sb::Data::Customdata','id'],
			['sb::Data::Entry','id'],
			['sb::Data::Image','id'],
			['sb::Data::Link','id'],
			['sb::Data::Message','id'],
			['sb::Data::Plugin','id'],
			['sb::Data::Session','id'],
			['sb::Data::Template','id'],
			['sb::Data::Trackback','id'],
			['sb::Data::User','id'],
			['sb::Data::Weblog','id'],
		];
		$change{'call'} = \&_change_from1;
	}
	return \%change;
}
sub _extract_name
{
	return lc( (split('::',$_[0]))[-1] );
}
# --------------------------------------
# // private functions - callbacks
sub _change_from1
{
	my $self = shift;
	my $driver = &_extract_name( ref($self->driver) );
	return 1 if ( $driver eq 'text' or $driver eq 'textold' );
	foreach my $class ( sb::Data->classes() )
	{
		my $temp = sb::Data->load($class,'sort'=>'id','order'=>1,'num'=>1); # order by file_id desc
		if ( $temp )
		{
			$self->driver->{'max'}->{lc( $class )} = $temp->id + 1;
		}
	}
	return $self->driver->save_ids();
}
1;
__END__
